﻿using Performance.DtoModels;
using Performance.EntityModels;
using Performance.Infrastructure;
using Performance.Services.AllotCompute;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Performance.Services
{
    /// <summary>
    /// 临床科室主任
    /// </summary>
    public class ComputeDirector : IAutoInjection
    {
        private BaiscNormService baiscNormService;
        public ComputeDirector(BaiscNormService baiscNormService)
        {
            this.baiscNormService = baiscNormService;
        }

        /// <summary>
        /// 临床科室主任、临床科室副主任、临床科室护士长 计算
        /// </summary>
        /// <param name="empolyeeList"></param>
        /// <param name="nurseList"></param>
        /// <param name="directorList"></param>
        /// <returns></returns>
        public List<ComputeResult> Compute(List<ComputeEmployee> empolyeeList, List<ComputeSource> nurseList, List<cof_director> directorList)
        {
            var typeArray = new[]
            {
                new { basic = EnumHelper.GetDescription(PerformanceType.临床科主任), key = "临床科室主任", deptment = AccountUnitType.临床科室.ToString()},
                new { basic = EnumHelper.GetDescription(PerformanceType.临床科副主任), key = "临床科室副主任", deptment = AccountUnitType.临床科室.ToString()},
                new { basic = EnumHelper.GetDescription(PerformanceType.医技主任), key = "医技主任", deptment = AccountUnitType.医技科室.ToString()},
                new { basic = EnumHelper.GetDescription(PerformanceType.医技副主任), key = "医技副主任", deptment = AccountUnitType.医技科室.ToString()},
                new { basic = EnumHelper.GetDescription(PerformanceType.临床护士长), key = "临床科室护士长", deptment = AccountUnitType.临床科室.ToString()},
                new { basic = EnumHelper.GetDescription(PerformanceType.临床护士长), key = "临床科室护士长", deptment = AccountUnitType.医技科室.ToString()},
            };

            List<ComputeResult> computeList = new List<ComputeResult>();
            foreach (var type in typeArray)
            {
                var needCompute = empolyeeList.Where(t => t.FitPeople == type.basic && t.AccountType == type.deptment);
                foreach (var item in needCompute)
                {
                    var resAccount = nurseList.FirstOrDefault(t => t.AccountingUnit == item.AccountingUnit);
                    var efficiency = directorList.FirstOrDefault(t => t.TypeName == "效率绩效" && t.JobTitle == type.key);
                    var scale = directorList.FirstOrDefault(t => t.TypeName == "规模绩效" && t.JobTitle == type.key);
                    var compute = new ComputeResult
                    {
                        AccountType = item.AccountType,
                        AccountingUnit = item.AccountingUnit,
                        EmployeeName = item.DoctorName,
                        FitPeople = item.FitPeople,
                        JobTitle = item.JobTitle,

                        Number = resAccount?.Number,
                        PerforTotal = resAccount?.PerforTotal,
                        Avg = resAccount?.Avg,
                        Efficiency = efficiency?.Value * resAccount?.Avg,
                        Scale = scale?.Value * resAccount?.PerforTotal,
                        Grant = item.Grant ?? 1,
                        ScoreAverageRate = item.ScoreAverageRate,
                        Punishment = item.Punishment,
                        OtherPerfor = item.OtherPerfor,
                        Adjust = item.Adjust,
                        Workload = item.Workload
                    };
                    //应发管理绩效
                    compute.ShouldGiveFee = (efficiency?.Value * resAccount?.Avg + scale?.Value * resAccount?.PerforTotal) * (item.Grant ?? 1);
                    //绩效合计
                    var perforSumFee = (compute.Avg + compute.Efficiency + compute.Scale) * (item.Grant ?? 1);
                    if (perforSumFee > (compute.Avg ?? 0) * 2.5m)
                        compute.PerforSumFee = (compute.Avg ?? 0) * 2.5m;
                    else
                        compute.PerforSumFee = perforSumFee;
                    //应发绩效
                    compute.GiveFee = compute.Avg * (item.Grant ?? 1) + (compute.Efficiency + compute.Scale) * (item.Grant ?? 1) * compute.ScoreAverageRate + (item.Punishment ?? 0) + (item.OtherPerfor ?? 0);
                    //实发绩效
                    compute.RealGiveFee = compute.GiveFee * (item.Adjust ?? 1m) - compute.ShouldGiveFee * item.ScoreAverageRate * (item.Grant ?? 1) * 0.1m;
                    computeList.Add(compute);
                }

            }
            return computeList;
        }

        /// <summary>
        /// 绩效标准计算
        /// </summary>
        /// <param name="computes"></param>
        /// <returns></returns>
        public List<res_baiscnorm> ComputeAvg(List<ComputeResult> computes)
        {
            var keyList = new[]
            {
                //绩效基数平均值项目
                new { type = "临床科室", reference = PerformanceType.临床科主任 },
                new { type = "临床科室", reference = PerformanceType.临床科副主任 },
                new { type = "医技科室", reference = PerformanceType.医技主任 },
                new { type = "医技科室", reference = PerformanceType.医技副主任 },
                new { type = "临床科室", reference = PerformanceType.临床护士长 },
            };

            var groupList = from cp in computes
                            join gp in keyList on new { type = cp.AccountType, reference = cp.FitPeople }
                            equals new { gp.type, reference = EnumHelper.GetDescription(gp.reference) }
                            select new { reference = gp.reference.ToString(), cp };

            var result = groupList.GroupBy(t => t.reference)
                .Select(s => new res_baiscnorm
                {
                    PositionName = s.Key,
                    TotelNumber = s.Count(),
                    TotelValue = s.Sum(t => t.cp.GiveFee),
                    AvgValue = s.Sum(t => t.cp.GiveFee) / s.Count()
                });
            return result.ToList();
        }

        /// <summary>
        /// 院领导、行政中层、行政工勤 平均值
        /// </summary>
        /// <param name="computes"></param>
        /// <returns></returns>
        public List<res_baiscnorm> ComputeOtherAvg(List<ComputeResult> computes)
        {
            var keyList = new[]
            {
                //行政绩效平均值项目
                new { type = "行政高层", groupname = EnumHelper.GetDescription(PerformanceType.TopManager) },
                new { type = "行政中层", groupname = EnumHelper.GetDescription(PerformanceType.MiddleManager) },
                new { type = "行政工勤", groupname = EnumHelper.GetDescription(PerformanceType.LogisticsWorker) },
            };

            var groupList = from cp in computes
                            join gp in keyList on cp.AccountType equals gp.type
                            select new { gp.groupname, cp };

            var result = groupList.GroupBy(t => t.groupname)
                .Select(s => new res_baiscnorm
                {
                    PositionName = s.Key,
                    TotelNumber = s.Count(),
                    TotelValue = s.Sum(t => t.cp.GiveFee),
                    AvgValue = s.Sum(t => t.cp.GiveFee) / s.Count()
                });
            return result.ToList();
        }

        /// <summary>
        /// 院领导、业务中层、工勤人员 计算
        /// </summary>
        /// <param name="empolyeeList"></param>
        /// <param name="baiscnormList"></param>
        /// <returns></returns>
        public List<ComputeResult> Compute(List<ComputeEmployee> empolyeeList, List<res_baiscnorm> baiscnormList, List<cof_workyear> workyearList)
        {
            List<ComputeResult> computeList = new List<ComputeResult>();
            var perforTypeArray = EnumHelper.GetItems<PerformanceType>();
            foreach (var type in EnumHelper.GetItems<AccountUnitType>())
            {
                var needCompute = empolyeeList.Where(t => t.AccountType == type.Description);
                if (!needCompute.Any()) continue;

                foreach (var item in needCompute)
                {
                    var compute = new ComputeResult
                    {
                        AccountType = item.AccountType,
                        AccountingUnit = item.AccountingUnit,
                        EmployeeName = item.DoctorName,
                        FitPeople = item.FitPeople,
                        Grant = item.Grant ?? 1,
                        WorkTime = item.WorkTime,
                        PostCoefficient = item.PostCoefficient,
                        Attendance = item.Attendance,
                        ScoreAverageRate = item.ScoreAverageRate,
                        Punishment = item.Punishment,
                        OtherPerfor = item.OtherPerfor,
                        JobTitle = item.JobTitle,
                        Adjust = item.Adjust,
                        Workload = item.Workload

                    };
                    decimal? baiscnorm = 1;
                    var perforTypeItem = perforTypeArray.FirstOrDefault(t => t.Description == item.FitPeople);
                    if (perforTypeItem != null)
                    {
                        var perforType = (PerformanceType)perforTypeItem.Value;
                        baiscnorm = baiscNormService.GetBaiscNorm(baiscnormList, perforType);
                    }
                    //年资系数
                    if (type.Value == (int)AccountUnitType.行政工勤 && item.WorkTime.HasValue && item.WorkTime.Value > new DateTime(1970, 1, 1))
                    {
                        var years = ((DateTime.Now.Year - item.WorkTime.Value.Year) * 12 + (DateTime.Now.Month - item.WorkTime.Value.Month)) / 12.0m;
                        var value = workyearList.FirstOrDefault(t => t.MinRange < years && years <= t.MaxRange)?.Value;
                        compute.WorkYear = value;
                    }
                    //添加参数计算
                    compute.BaiscNormValue = baiscnorm * (item.FitPeopleRatio ?? 1);
                    //应发绩效
                    compute.GiveFee = baiscnorm * compute.PostCoefficient * compute.Attendance * compute.ScoreAverageRate * (compute.WorkYear ?? 1) + (compute.OtherPerfor ?? 0) + (item.Workload ?? 0) + (compute.Punishment ?? 0);
                    //实发绩效
                    var adjust = item.Adjust ?? 1m;
                    compute.RealGiveFee = compute.GiveFee * adjust * (item.Grant ?? 1);
                    computeList.Add(compute);
                }
            }
            return computeList;
        }
    }


}
